#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "crandom.h"
#include "field.h"
#include "config.h"
#include "main.h"

static spot_t field[FIELD_HEIGHT][FIELD_WIDTH];

spot_t field_get(int y, int x)
{
	assert(y >= 1);
	assert(x >= 1);
	assert(y <= FIELD_HEIGHT);
	assert(x <= FIELD_WIDTH);
	return field[y-1][x-1];
}

void field_set(int y, int x, spot_t spot)
{
	assert(y >= 1);
	assert(x >= 1);
	assert(y <= FIELD_HEIGHT);
	assert(x <= FIELD_WIDTH);
	field[y-1][x-1] = spot;
}

const char *symbols = "-|/\\*+xv^><$#@.";

#define SEG_WIDTH (FIELD_WIDTH/2)
#define SEG_HEIGHT (FIELD_HEIGHT/2)

static void loadsegment(const char *filename, int ay, int ax)
{
	int y = 0, x = 0;
	int c;

	FILE *fp = fopen(filename, "r");
	if (fp == NULL) abort_with_error("Can't open file %s", filename);
	while ((c = getc(fp)) != EOF)
	{
		char *place = strchr(symbols, c);
		if (place == NULL) continue;
		field_set(y+ay, x+ax, place - symbols);
		x++;
		if (x == SEG_WIDTH)
		{
			x = 0;
			y++;
			if (y == SEG_HEIGHT)
				break;
		}
	}
	if (y < SEG_HEIGHT)
		abort_with_error("Level segment too small in %s", filename);
	fclose(fp);
}

#define NUMSEGS 15

void field_load(void)
{
	int y, x, segnum;
	char filename[1100];
#ifndef NO_LOG
	FILE *log;

	log = fopen("log.txt", "a");
	if (log != NULL)
	{
		fprintf(log, "Making a new level with the following "
			"segments:\n");
	}
#endif

	for (y = 1; y <= FIELD_HEIGHT; y += SEG_HEIGHT)
		for (x = 1; x <= FIELD_WIDTH; x += SEG_WIDTH)
		{
#ifdef ONESEG /* Test by loading all one particular segment */
			segnum = ONESEG;
#else
			segnum = rnd(NUMSEGS)+1;
#endif

#ifndef NO_LOG
			if (log != NULL)
				fprintf(log, "%d\n", segnum);
#endif

			snprintf(filename, sizeof filename,
				"%sSeg-%d", SHAREPATH, segnum);
			loadsegment(filename, y, x);
		}

#ifndef NO_LOG
	if (log != NULL)
	{
		for (y = 1; y <= FIELD_HEIGHT; y++)
		{
			for (x = 1; x <= FIELD_WIDTH; x++)
				putc(symbols[field_get(y,x)], log);
			putc('\n', log);
		}
	}
#endif

#ifndef NO_LOG
	if (log != NULL) fclose(log);
#endif
}
